﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Netzlast
{
    abstract class Netzelement
    {
        public abstract double BestimmeLeistung(DateTime wann); // in Watt; Einspeisung negativ, Verbrauch positiv
    }

    abstract class Verbraucher : Netzelement
    {
    }

    class Glühlampe : Verbraucher
    {
        static Random zufallsgenerator = new Random(42);
        double[] lastgang = new double[24 * 60];

        public Glühlampe(double wattzahl)
        {
            int minuteDesNächstenSchaltvorgangs = 0;
            bool istAn = false;
            for (int minute = 0; minute < 24 * 60; minute++)
            {
                if(minute == minuteDesNächstenSchaltvorgangs)
                {
                    istAn = !istAn;
                    minuteDesNächstenSchaltvorgangs = minute + zufallsgenerator.Next(15, 45);
                }
                if(istAn)
                {
                    int stunde = minute / 60;
                    if(stunde >= 6 && stunde < 8 || stunde >= 17 && stunde < 22)
                    {
                        lastgang[minute] = wattzahl;
                    }
                }
            }
        }

        public override double BestimmeLeistung(DateTime wann)
        {
            return lastgang[60 * wann.Hour + wann.Minute];
        } 
    }

    class Waschmaschine : Verbraucher
    {
        public override double BestimmeLeistung(DateTime wann)
        {
            if (wann.Hour == 18)
            {
                return wann.Minute < 20 ? 2000.0 : 500.0;
            }
            return 0.0;
        }
    }

    abstract class Erzeuger : Netzelement
    {
    }

    class Photovoltaikanlage : Erzeuger
    {
        static Random zufallsgenerator = new Random(43);
        double[] tagesgang = new double[24 * 60];

        public Photovoltaikanlage(double kWp)
        {
            for (int minute = 0; minute < 24 * 60; minute++)
            {
                double einstrahlung = 1.0 - 2.0 * Math.Cos(2.0 * Math.PI * minute / (24 * 60));
                if(einstrahlung > 0)
                {
                    // Unplausibel: Die Änderung von Minute zu Minute ist mit dieser Formel noch rein zufällig.
                    double bewölkung = 0.1 + 0.7 * zufallsgenerator.NextDouble();
                    tagesgang[minute] = 0.8 * (1.0 - bewölkung) * einstrahlung * 1000.0 * kWp;
                }
            }
        }

        public override double BestimmeLeistung(DateTime wann)
        {
            return -tagesgang[60 * wann.Hour + wann.Minute];
        } 
    }

    class MiniWKA : Erzeuger
    {
        static Random zufallsgenerator = new Random(44);
        double[] tagesgang = new double[24 * 60];

        const double lambda = 1.0 / 6.0;
        const double k = 2.0;

        public MiniWKA(double durchmesser, double kWp)
        {
            for (int minute = 0; minute < 24 * 60; minute++)
            {
                // Diese Formeln hier sind etwas übertrieben, schon allein,
                // weil sie nicht berücksichtigen, dass die Änderung
                // von Minute zu Minute nicht rein zufällig ist.
                double windgeschwindigkeit = 1.0 / lambda * Math.Pow(-Math.Log(0.001 + 0.998 * zufallsgenerator.NextDouble()), 1.0 / k);
                double leistungsdichte = 1.2 / 2.0 * Math.Pow(windgeschwindigkeit, 3.0);
                double leistung = 0.4 * Math.PI * durchmesser * durchmesser / 4.0 * leistungsdichte;
                tagesgang[minute] = Math.Min(leistung, 1000.0 * kWp);
            }
        }

        public override double BestimmeLeistung(DateTime wann)
        {
            return -tagesgang[60 * wann.Hour + wann.Minute];
        } 
    }
}
